# -*- coding: utf-8 -*-
import os
from vcr import VCR

from django.conf import settings
from django.test.client import Client
from django.test import TestCase
from django.contrib.auth import authenticate, login
from django.utils.importlib import import_module
from django.http import HttpRequest

from identity_client import PERSISTENCE_MODULE


__all__ = [
    'MyfcIDTestClient', 'MyfcIDTestCase', 'MyfcIDAPITestCase', 'use_cassette'
]


def use_cassette(*args, **kwargs):
    return VCR(
        cassette_library_dir = os.path.join(os.path.dirname(__file__), 'cassettes', 'api_client'),
        match_on = ['url', 'method', 'headers', 'body'],
        record_mode = 'new_episodes',
    ).use_cassette(*args, **kwargs)


class MyfcIDTestClient(Client):

    def login(self, **credentials):
        """
        Sets the Client to appear as if it has successfully logged into a site.

        Returns True if login is possible; False if the provided credentials
        are incorrect, or the user is inactive, or if the sessions framework is
        not available.
        """
        user = authenticate(**credentials)
        if user:
            engine = import_module(settings.SESSION_ENGINE)

            # Create a fake request to store login details.
            request = HttpRequest()
            if self.session:
                request.session = self.session
            else:
                request.session = engine.SessionStore()
            login(request, user)

            # Save the session values.
            request.session.save()

            # Set the cookie to represent the session.
            session_cookie = settings.SESSION_COOKIE_NAME
            self.cookies[session_cookie] = request.session.session_key
            cookie_data = {
                'max-age': None,
                'path': '/',
                'domain': settings.SESSION_COOKIE_DOMAIN,
                'secure': settings.SESSION_COOKIE_SECURE or None,
                'expires': None,
            }
            self.cookies[session_cookie].update(cookie_data)

            return True
        else:
            return False


class MyfcIDTestCase(PERSISTENCE_MODULE.TestCase):

    userdata = {
        'login' : 'cesar',
        'fullname' : 'Cesar Lustosa',
        'email' : 'cesar@lustosa.net',
        'activation_key': 'Ad6e1161446611529d976dcfb9027eb55',
        'id' : 275310,
        'password': 'e10adc3949ba59abbe56e057f20f883e',
    }

    def __call__(self, result=None):
        """
        Wrapper around default __call__ method to perform common Django test
        set up. This means that user-defined Test Cases aren't required to
        include a call to super().setUp().
        """
        self.client = MyfcIDTestClient()
        try:
            self._pre_setup()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            import sys
            result.addError(self, sys.exc_info())
            return
        super(PERSISTENCE_MODULE.TestCase, self).__call__(result)
        try:
            self._post_teardown()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            import sys
            result.addError(self, sys.exc_info())
            return


class MyfcIDAPITestCase(MyfcIDTestCase):

    http_method = None
    url_name = None
    url = None

    def make_request(self, url, *args, **kwargs):
        client_method = getattr(self.client, self.http_method.lower())
        return client_method(url, *args, **kwargs)


    def test_unauthorized_service(self):
        self.service.acl = []
        self.service.save()

        response = self.make_request(self.url, {},
            HTTP_ACCEPT = 'application/json',
            HTTP_AUTHORIZATION = self.auth,
        )

        self.assertEqual(response.status_code, 403)


    def test_unauthenticated_service(self):

        response = self.make_request(self.url, {},
            HTTP_ACCEPT = 'application/json',
        )

        self.assertEqual(response.status_code, 403)



# -*- coding: utf-8 -*-
from httplib2 import HttpLib2Error
from oauth2 import Token
from datetime import datetime, timedelta
import json

from mock import Mock, patch

from django.utils import translation
from django.utils.importlib import import_module
from django.conf import settings
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.contrib.auth import REDIRECT_FIELD_NAME

from mock_helpers import *
from identity_client.models import Identity
from identity_client.utils import get_account_module
from identity_client.sso.client import SSOClient

__all__ = ['SSOFetchRequestTokenView', 'AccessUserData']

mocked_user_json = """{
    "last_name": "Doe",
    "services": ["financedesktop"],
    "timezone": null,
    "nickname": null,
    "first_name": "John",
    "language": "en",
    "country": null,
    "cpf": null,
    "gender": null,
    "birth_date": "2010-05-04",
    "email": "jd@123.com",
    "uuid": "16fd2706-8baf-433b-82eb-8c7fada847da",
    "is_active": true,
    "accounts": [
        {
            "plan_slug": "plus",
            "name": "Pessoal",
            "roles": ["owner"],
            "url": "http://192.168.1.48:8000/organizations/api/accounts/e823f8e7-962c-414f-b63f-6cf439686159/",
            "expiration": "%s",
            "external_id": null,
            "uuid": "e823f8e7-962c-414f-b63f-6cf439686159"
        },
        {
            "plan_slug": "max",
            "name": "Myfreecomm",
            "roles": ["owner"],
            "url": "http://192.168.1.48:8000/organizations/api/accounts/b39bad59-94af-4880-995a-04967b454c7a/",
            "expiration": "%s",
            "external_id": null,
            "uuid": "b39bad59-94af-4880-995a-04967b454c7a"
        }
    ]
}""" % (
    (datetime.today() + timedelta(days=1)).strftime('%Y-%m-%d'),
    (datetime.today() + timedelta(days=30)).strftime('%Y-%m-%d'),
)

mocked_user_corrupted = """{
    "last_name": "Doe",
    "services": ["financedesktop"],
    "timezone": null,
    "nickname": null,
    "first_name": "John",
    "language": n
"""

request_token_session = {OAUTH_REQUEST_TOKEN: OAUTH_REQUEST_TOKEN_SECRET}
dummy_access_token = Token(OAUTH_ACCESS_TOKEN, OAUTH_ACCESS_TOKEN_SECRET)
mocked_user_dict = json.loads(mocked_user_json)


class SSOFetchRequestTokenView(TestCase):

    @patch_httplib2(Mock(return_value=mocked_request_token()))
    def test_request_token_success(self):
        response = self.client.get(reverse('sso_consumer:request_token'), {})

        authorization_url = '%(HOST)s/%(AUTHORIZATION_PATH)s' % settings.PASSAPORTE_WEB

        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response['Location'],
            authorization_url + '?oauth_token=' + OAUTH_REQUEST_TOKEN
        )

        session = self.client.session

        self.assertTrue('request_token' in session)
        self.assertTrue(OAUTH_REQUEST_TOKEN in session['request_token'])
        self.assertEqual(
            session['request_token'][OAUTH_REQUEST_TOKEN],
            OAUTH_REQUEST_TOKEN_SECRET
        )

    @patch_httplib2(Mock(return_value=mocked_response(401, 'invalid token')))
    def test_request_token_fails_on_invalid_token(self):
        response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 500)

    @patch_httplib2(Mock(return_value=mocked_response(200, 'invalid_request_token')))
    def test_handles_malformed_request_token(self):
        response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 500)

    @patch_httplib2(Mock(side_effect=HttpLib2Error))
    def test_request_token_fails_on_broken_oauth_provider(self):

        response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 502)

    @patch_httplib2(Mock(return_value=mocked_response(200, 'corrupted_data')))
    def test_request_token_fails_on_corrupted_data(self):
        response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 500)


class AccessUserData(TestCase):

    @patch_httplib2(Mock(return_value=mocked_request_token()))
    def setUp(self):
        response = self.client.get(reverse('sso_consumer:request_token'), {})
        self.assertEquals(response.status_code, 302)


    def _get_real_session(self, client):
        if 'django.contrib.sessions' in settings.INSTALLED_APPS:
            engine = import_module(settings.SESSION_ENGINE)
            cookie = client.cookies.get(settings.SESSION_COOKIE_NAME, None)
            return engine.SessionStore(cookie and cookie.value or None)


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    def test_access_user_data_successfuly(self):

        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].endswith(settings.LOGIN_REDIRECT_URL))
        self.assertNotEqual(self.client.session.get('user_data'), None)


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    def test_next_url_may_be_read_from_session(self):
        session = self._get_real_session(self.client)
        session[REDIRECT_FIELD_NAME] = '/oauth-protected-view/'
        session.save()

        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].endswith('/oauth-protected-view/'))


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    def test_user_language_is_added_to_session(self):
        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 302)
        # Django < 1.7 does not have translation.LANGUAGE_SESSION_KEY
        LANGUAGE_SESSION_KEY = getattr(translation, 'LANGUAGE_SESSION_KEY', 'django_language')
        self.assertEqual(self.client.session.get(LANGUAGE_SESSION_KEY), 'en')


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    def test_default_next_url_is_LOGIN_REDIRECT_URL(self):
        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].endswith(settings.LOGIN_REDIRECT_URL))


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(side_effect=HttpLib2Error))
    def test_access_user_data_fails_if_myfc_id_is_down(self):
        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 502)


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_corrupted)))
    def test_access_user_data_fails_if_corrupted_data_is_received(self):
        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 500)


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    @patch('oauth2.Request.sign_request')
    def test_oauth_request_user_data_is_correctly_signed(self, sign_request_mock):
        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertTrue(isinstance(sign_request_mock.call_args[0][2], Token))


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    def test_authentication_creates_local_user(self):
        Identity.objects.all().delete()

        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
        )

        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].endswith(settings.LOGIN_REDIRECT_URL))

        self.assertEqual(Identity.objects.count(), 1)


    @patch.object(SSOClient, 'fetch_access_token', Mock(return_value=dummy_access_token))
    @patch_httplib2(Mock(return_value=mocked_response(200, mocked_user_json)))
    @patch.object(settings, 'SERVICE_ACCOUNT_MODULE', 'identity_client.ServiceAccount')
    def test_authentication_creates_local_user_accounts(self):
        serviceAccountModel = get_account_module()
        serviceAccountModel.objects.all().delete()

        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': OAUTH_REQUEST_TOKEN,
                'oauth_verifier': 'niceverifier'
            },
      )

        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].endswith(settings.LOGIN_REDIRECT_URL))

        self.assertEqual(serviceAccountModel.objects.count(), 2)



# -*- coding: utf-8 -*-
import logging
import json
from datetime import datetime, date

import requests
from passaporte_web.main import Application, Identity as RemoteIdentity, ServiceAccount as RemoteServiceAccount, AccountMembers, AccountMember
from django.conf import settings

from identity_client.decorators import handle_api_exceptions, handle_api_exceptions_with_form

__all__ = ['APIClient']

# TODO: 
# - fetch_application_accounts
# - perfil
# - notificações
#   - como tratar a paginação?


class APIClient(object):

    api_host = settings.PASSAPORTE_WEB['HOST']
    api_user = settings.PASSAPORTE_WEB['CONSUMER_TOKEN']
    api_password = settings.PASSAPORTE_WEB['CONSUMER_SECRET']
    profile_api = settings.PASSAPORTE_WEB['PROFILE_API']
    registration_api = settings.PASSAPORTE_WEB['REGISTRATION_API']

    pweb = requests.Session()
    pweb.auth = (api_user, api_password)
    pweb.headers.update({
        'cache-control': 'no-cache',
        'content-length': '0',
        'content-type': 'application/json',
        'accept': 'application/json',
        'user-agent': 'myfc_id client',
    })
    _application = None

    @classmethod
    def get_application(cls):
        is_same_app = lambda app: (app.token == cls.api_user) and \
             (app.secret == cls.api_password) and \
             (app.host == cls.api_host)

        if (cls._application is None) or not is_same_app(cls._application):
            cls._application = Application(host=cls.api_host, token=cls.api_user, secret=cls.api_password)

        return cls._application

    @classmethod
    @handle_api_exceptions_with_form
    def invoke_registration_api(cls, form):
        current_app = cls.get_application()
        logging.info(u'Trying to create user "{0}"'.format(form.cleaned_data.get('email', 'no email set')))
        user = current_app.users.create(**form.cleaned_data)
        return user.response.status_code, user.response.json()

    @classmethod
    @handle_api_exceptions
    def fetch_identity_data(cls, **kwargs):
        current_app = cls.get_application()
        logging.info(u'Trying to fetch user with params "{0}"'.format(kwargs))
        user = current_app.users.get(**kwargs)
        return user.response.status_code, user.response.json()

    @classmethod
    @handle_api_exceptions_with_form
    def update_user_api(cls, form, api_path):
        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        logging.info(u'Loading information for user identified by "{0}"'.format(url))
        remote_user = RemoteIdentity.load(url, token=cls.api_user, secret=cls.api_password)

        remote_user.resource_data.update(form.cleaned_data)
        logging.info(u'Updating information for user identified by "{0}"'.format(url))
        remote_user = remote_user.save()
        return remote_user.response.status_code, remote_user.response.json()

    @classmethod
    @handle_api_exceptions
    def fetch_association_data(cls, api_path):

        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        logging.info('fetch_association_data: Making request to %s', url)
        response = cls.pweb.get(url)

        if response.status_code != 200:
            response.raise_for_status()
            raise requests.exceptions.HTTPError('Unexpected response', response=response)

        return response.status_code, response.json()

    @classmethod
    @handle_api_exceptions
    def update_association_data(cls, new_data, api_path):

        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        logging.info('update_association_data: Making request to %s', url)

        association_data = json.dumps(new_data)
        response = cls.pweb.put(
                url,
                headers={'content-length': str(len(association_data))},
                data=association_data
        )

        if response.status_code != 200:
            response.raise_for_status()
            raise requests.exceptions.HTTPError('Unexpected response', response=response)

        return response.status_code, response.json()

    @classmethod
    @handle_api_exceptions
    def fetch_user_accounts(cls, uuid, **kwargs):
        current_app = cls.get_application()
        logging.info(u'Trying to fetch user with uuid "{0}"'.format(uuid))
        user = current_app.users.get(uuid=uuid)

        logging.info(u'Fetching all accounts for user "{0}"'.format(user.uuid))
        kwargs['load_options'] = False
        user_accounts = [item.resource_data for item in user.accounts.all(**kwargs)]

        return 200, user_accounts

    @classmethod
    @handle_api_exceptions
    def fetch_account_data(cls, account_uuid):
        current_app = cls.get_application()
        logging.info(u'Trying to account with uuid "{0}"'.format(account_uuid))
        account = current_app.accounts.get(account_uuid)
        return account.response.status_code, account.response.json()

    @classmethod
    @handle_api_exceptions
    def create_user_account(cls, uuid, plan_slug, account_uuid=None, account_name=None, expiration=None):
        account_data = {'plan_slug': plan_slug, 'expiration': expiration}
        if account_uuid:
            account_data['uuid'] = account_uuid
        elif account_name:
            account_data['name'] = account_name
        else:
            raise ValueError("Either 'account_uuid' or 'account_name' must be given")

        current_app = cls.get_application()
        logging.info(u'Trying to fetch user with uuid "{0}"'.format(uuid))
        user = current_app.users.get(uuid=uuid)

        logging.info(u'Creating account for user "{0}" with "{1}"'.format(uuid, account_data))
        new_account = user.accounts.create(**account_data)

        return new_account.response.status_code, new_account.response.json()

    @classmethod
    @handle_api_exceptions
    def update_account_data(cls, plan_slug, expiration, api_path):
        if isinstance(expiration, datetime):
            raise TypeError(u'expiration must be a date instance or None')
        elif isinstance(expiration, date):
            expiration = expiration.isoformat()
        elif expiration is not None:
            raise TypeError(u'expiration must be a date instance or None')

        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        logging.info(u'Loading information for service account identified by "{0}"'.format(url))
        remote_account = RemoteServiceAccount.load(url, token=cls.api_user, secret=cls.api_password)
        remote_account.resource_data.update({'plan_slug': plan_slug, 'expiration': expiration})

        logging.info(u'Updating information for service account identified by "{0}"'.format(url))
        remote_account = remote_account.save()

        return remote_account.response.status_code, remote_account.response.json()

    @classmethod
    @handle_api_exceptions
    def add_account_member(cls, user_uuid, roles, api_path):
        if not isinstance(roles, list):
            raise TypeError(u"roles must be a list")

        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        account_members = AccountMembers(url, token=cls.api_user, secret=cls.api_password)
        
        logging.info(u'Adding user with uuid "{0}" to account members identified by url "{1}"'.format(user_uuid, url))
        member = account_members.create(identity=user_uuid, roles=roles)

        return member.response.status_code, member.response.json()

    @classmethod
    @handle_api_exceptions
    def update_member_roles(cls, roles, api_path):
        if not isinstance(roles, (list, set)):
            raise TypeError(u"roles must be a list")

        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        logging.info(u'Loading information for account member identified by "{0}"'.format(url))
        account_member = AccountMember.load(url, token=cls.api_user, secret=cls.api_password)
        account_member.roles = roles

        logging.info(u'Updating information for account member identified by "{0}"'.format(url))
        account_member = account_member.save()

        return account_member.response.status_code, account_member.response.json()

    @classmethod
    @handle_api_exceptions
    def remove_account_member(cls, api_path):
        if api_path.startswith(cls.api_host):
            url = api_path
        else:
            url = "{0}{1}".format(cls.api_host, api_path)

        account_member = AccountMember.load(url, token=cls.api_user, secret=cls.api_password)
        logging.info(u'Removing account member identified by "{0}"'.format(url))
        account_member.delete()

        return 204, ''



#coding UTF-8
from django.test import TestCase

from identity_client.forms import RegistrationForm


__all__ = ['TestApiRegistrationForm']

def create_post(**kwargs):
    post_data = {
        'password':'1234567',
        'password2':'1234567',
        'email':'kiti@come.com',
        'tos':True,
    }
    post_data.update(kwargs)
    return post_data

class TestApiRegistrationForm(TestCase):

    def test_validation_error_on_no_password_match(self):
        form = RegistrationForm(create_post(password='123456', password2='456321'))
        self.assertFalse(form.is_valid())
        #non-field error
        self.assertTrue('__all__' in form.errors)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding model 'Identity'
        db.create_table('identity_client_identity', (
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=50, null=True)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=100, null=True)),
            ('uuid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=36)),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(unique=True, max_length=75)),
            ('last_login', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('identity_client', ['Identity'])
    
    
    def backwards(self, orm):
        
        # Deleting model 'Identity'
        db.delete_table('identity_client_identity')
    
    
    models = {
        'identity_client.identity': {
            'Meta': {'object_name': 'Identity'},
            'email': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '36'})
        }
    }
    
    complete_apps = ['identity_client']



# -*- coding: utf-8 -*-
import logging
import json
import requests
from django.contrib.auth.models import AnonymousUser
from django.conf import settings

from identity_client.models import Identity
from identity_client.signals import pre_identity_authentication
from identity_client.decorators import handle_api_exceptions


class MyfcidAPIBackend(object):
    """
    Authenticates a user using the Myfc ID API.
    """
    supports_anonymous_user = False
    supports_inactive_user = False
    supports_object_permissions = False


    def authenticate(self, email=None, password=None, username=None):
        if email and username:
            raise TypeError
        elif username:
            email = username

        identity = None

        # Fetch user data
        status_code, user_data, error = self.fetch_user_data(email, password)

        if user_data:
            logging.info(u'User %s (%s) authenticated', email, user_data['uuid'])
            identity = self.create_local_identity(user_data)
        else:
            logging.info(u'Failed to authenticate user %s', email)

        return identity


    def create_local_identity(self, user_data):
        # Create an updated Identity instance for this user
        identity, created = Identity.objects.get_or_create(uuid=user_data['uuid'])
        self._update_user(identity, user_data)

        # Set this user's backend
        identity.backend = "%s.%s" % (MyfcidAPIBackend.__module__, 'MyfcidAPIBackend')

        # Append additional user data to a temporary attribute
        identity.user_data = user_data

        pre_identity_authentication.send_robust(
            sender='identity_client.MyfcidAPIBackend.backend',
            identity = identity,
            user_data = user_data,
        )
        return identity


    def get_user(self, user_id):
        try:
            user = Identity.objects.get(id=user_id)
        except Exception: # Usuário não existe ou formato do id incorreto (mongodb)
            user = None

        return user


    def _update_user(self, user, user_data):
        user.email = user_data['email']
        user.first_name = user_data['first_name']
        user.last_name = user_data['last_name']
        user.is_active = True
        user.save()


    @handle_api_exceptions
    def fetch_user_data(self, user, password):

        # Build the API uri
        uri = "{0[HOST]}/{0[AUTH_API]}".format(settings.PASSAPORTE_WEB)

        headers = {
            'content-type': 'application/json',
            'cache-control': 'no-cache',
            'user-agent': 'myfc_id client'
        }

        # Request the data
        response = requests.get(uri, auth=(user, password), headers=headers)
        logging.info(
            u'Auth response: status=%s, content=%s',
            response.status_code, response.text
        )

        # If the request is successful, read response data
        if response.status_code != 200:
            response.raise_for_status()
            raise requests.exceptions.HTTPError('Unexpected response', response=response)

        return response.status_code, response.json()


def get_user(userid=None):
    """
    Returns a User object from an id (User.id). Django's equivalent takes
    request, but taking an id instead leaves it up to the developer to store
    the id in any way they want (session, signed cookie, etc.)
    """
    if not userid:
        return AnonymousUser()
    return MyfcidAPIBackend().get_user(userid) or AnonymousUser()



#coding: utf-8
import oauth2 as oauth
from mock import Mock, patch

from django.utils.importlib import import_module
from django.test import TestCase
from django.http import HttpRequest
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from mock_helpers import *

from identity_client.sso.client import SSOClient


__all__ = ['SSOClientAuthorize', 'SSOClientRequestToken', 'SSOClientAccessToken']


def create_signed_oauth_request(consumer, sso_client):

    plaintext_signature = oauth.SignatureMethod_PLAINTEXT()

    oauth_request = oauth.Request.from_consumer_and_token(consumer,
                                http_url=sso_client.request_token_url,
                                parameters={'scope':'sso-sample'})

    oauth_request.sign_request(plaintext_signature, consumer, None)

    #XXX: nao sabemos como passar o callback sem hack
    oauth_request['oauth_callback'] = 'http://callback.example.com'

    return oauth_request

def build_access_token_request(oauth_token, oauth_verifier):
    sso_client = SSOClient()

    secret = OAUTH_REQUEST_TOKEN_SECRET
    token = oauth.Token(key=oauth_token, secret=secret)
    token.set_verifier(oauth_verifier)

    consumer = oauth.Consumer(settings.PASSAPORTE_WEB['CONSUMER_TOKEN'],
                               settings.PASSAPORTE_WEB['CONSUMER_SECRET'])
    signature_method_plaintext = oauth.SignatureMethod_PLAINTEXT()
    oauth_request = oauth.Request.from_consumer_and_token(consumer, token=token,
                 http_url=sso_client.access_token_url, parameters={'scope':'sso-sample'})

    return oauth_request


class SSOClientAuthorize(TestCase):

    def setUp(self):
        self.sso_client = SSOClient()

    def _get_real_session(self, client):
        if 'django.contrib.sessions' in settings.INSTALLED_APPS:
            engine = import_module(settings.SESSION_ENGINE)
            cookie = client.cookies.get(settings.SESSION_COOKIE_NAME, None)
            return engine.SessionStore(cookie and cookie.value or None)

    @patch.object(SSOClient, 'fetch_request_token', Mock(return_value=oauth.Token(key='key', secret='secret')))
    def test_authorize_invokes_fetch_request_token(self):
        self.sso_client.fetch_request_token.reset_mock()

        request = HttpRequest()
        request.session = self._get_real_session(self.client)
        authorization_url = self.sso_client.authorize(request)

        self.assertTrue(self.sso_client.fetch_request_token.called)
        self.assertEquals(self.sso_client.fetch_request_token.call_count, 1)
        self.assertEquals(self.sso_client.fetch_request_token.call_args, ((), {}))



    @patch.object(SSOClient, 'fetch_request_token', Mock(return_value=oauth.Token(key='key', secret='secret')))
    def test_authorize_stores_request_token_in_session(self):
        self.sso_client.fetch_request_token.reset_mock()

        request = HttpRequest()
        request.session = self._get_real_session(self.client)
        authorization_url = self.sso_client.authorize(request)

        self.assertTrue('request_token' in request.session.keys())
        self.assertEquals(request.session['request_token'], {'key': 'secret'})


    @patch.object(SSOClient, 'fetch_request_token', Mock(return_value=oauth.Token(key='key', secret='secret')))
    def test_authorize_stores_next_url_in_session(self):
        self.sso_client.fetch_request_token.reset_mock()

        request = HttpRequest()
        request.GET = {REDIRECT_FIELD_NAME: '/oauth-protected-view/'}
        request.session = self._get_real_session(self.client)
        authorization_url = self.sso_client.authorize(request)

        self.assertTrue(REDIRECT_FIELD_NAME in request.session.keys())
        self.assertEquals(request.session[REDIRECT_FIELD_NAME], '/oauth-protected-view/')


class SSOClientRequestToken(TestCase):

    def setUp(self):
        self.sso_client = SSOClient()

    @patch_httplib2(Mock(return_value=mocked_request_token()))
    def test_fetch_request_token_succeeded(self):
        request_token = self.sso_client.fetch_request_token()

        self.assertEqual(OAUTH_REQUEST_TOKEN, request_token.key)
        self.assertEqual(OAUTH_REQUEST_TOKEN_SECRET, request_token.secret)

    @patch_httplib2(Mock(return_value=mocked_response(401, 'invalid token')))
    def test_fetch_request_token_fails_on_invalid_token(self):
        self.assertRaises(
            AssertionError, self.sso_client.fetch_request_token
        )

    @patch_httplib2(Mock(side_effect=AttributeError))
    def test_http_exceptions_are_not_handled(self):
        self.assertRaises(AttributeError, self.sso_client.fetch_request_token)


class SSOClientAccessToken(TestCase):

    def setUp(self):
        self.sso_client = SSOClient()

    @patch_httplib2(Mock(return_value=mocked_access_token()))
    def test_fetch_access_token_succeeded(self):
        access_token = self.sso_client.fetch_access_token()

        self.assertEqual(OAUTH_ACCESS_TOKEN, access_token.key)

    @patch_httplib2(Mock(return_value=mocked_response(401, 'invalid verifier')))
    def test_fetch_access_token_fails_on_invalid_verifier(self):
        self.assertRaises(
            AssertionError, self.sso_client.fetch_access_token
        )

    @patch_httplib2(Mock(return_value=mocked_response(401, 'invalid token')))
    def test_fetch_access_token_fails_on_invalid_token(self):
        self.assertRaises(
            AssertionError, self.sso_client.fetch_access_token
        )



# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

__all__ = ['PERSISTENCE_MODULE']

module_name = 'identity_client.%s' % settings.PERSISTENCE_STRATEGY

try:
    PERSISTENCE_MODULE = __import__(
        module_name,
        fromlist=['identity_client']
    )
except ImportError:
    raise ImproperlyConfigured(
        'settings.PERSISTENCE_MODULE: %s could not be imported' % module_name
    )

new_settings = getattr(PERSISTENCE_MODULE, 'settings')
for attr_name in dir(new_settings):
    if not attr_name.isupper():
        continue

    attr_value = getattr(new_settings, attr_name)
    setattr(settings, attr_name, attr_value)

import handlers



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.shortcuts import render_to_response
from django.contrib.sites.models import Site, RequestSite
from django.http import HttpResponseRedirect
from django.template import RequestContext
from django.views.decorators.cache import never_cache
from django.utils import translation

from identity_client.backend import MyfcidAPIBackend
from identity_client.forms import RegistrationForm
from identity_client.decorators import required_method, sso_login_required
from identity_client.forms import IdentityAuthenticationForm as AuthenticationForm
from identity_client.utils import prepare_form_errors, get_account_module
from identity_client.client_api_methods import APIClient

__all__ = ["new_identity", "register", "login", "show_login", "login_user", "list_accounts"]

@required_method("GET")
@never_cache
def new_identity(request,template_name='registration_form.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          registration_form=RegistrationForm, **kwargs):

    if request.user.is_authenticated():
        return redirect_logged_user(request, redirect_field_name)

    form = registration_form()
    return handle_redirect_to(
        request, template_name, redirect_field_name, form, **kwargs
    )


@required_method("POST")
def register(request, template_name='registration_form.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          registration_form=RegistrationForm, **kwargs):

    form = registration_form(data=request.POST)
    if form.is_valid():
        # Registro
        status, content, form = APIClient.invoke_registration_api(form)
        if status in (200, 201):
            user = MyfcidAPIBackend().create_local_identity(content)
            login_user(request, user)
            return redirect_logged_user(request, redirect_field_name)

    return handle_redirect_to(
        request, template_name, redirect_field_name, form, **kwargs
    )


@required_method("GET")
@never_cache
def show_login(request, template_name='login.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          authentication_form=AuthenticationForm, **kwargs):

    if request.user.is_authenticated():
        return redirect_logged_user(request, redirect_field_name)

    form = authentication_form()
    return handle_redirect_to(
        request, template_name, redirect_field_name, form, **kwargs
    )


@required_method("POST")
def login(request, template_name='login.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          authentication_form=AuthenticationForm, **kwargs):

    form = authentication_form(data=request.POST)
    if form.is_valid():
        user = form.get_user()
        login_user(request, user)
        result = redirect_logged_user(request, redirect_field_name)
    else:
        result = handle_redirect_to(
            request, template_name, redirect_field_name, form, **kwargs
        )

    return result

@sso_login_required
def list_accounts(request):
    identity = request.user
    serviceAccount = get_account_module()
    if serviceAccount is None:
        accounts = []
    else:
        accounts = serviceAccount.for_identity(identity, include_expired=True)

    status, remote_accounts, error = APIClient.fetch_user_accounts(identity.uuid)
    context = {
        'accounts': accounts,
        'remote_accounts': remote_accounts,
        'error': error,
    }

    return render_to_response('accounts_list.html',
        RequestContext(request, context))


#======================================


def login_user(request, user):
    # Efetuar login
    from django.contrib.auth import login as django_login
    django_login(request, user)

    # Adicionar dados adicionais do usuário à sessão
    try:
        request.session['user_data'] = user.user_data
        del(user.user_data)
    except AttributeError:
        request.session['user_data'] = {}

    user_language = request.session['user_data'].get('language')
    if user_language in [code for (code, label) in settings.LANGUAGES]:
        # Django < 1.7 does not have translation.LANGUAGE_SESSION_KEY
        LANGUAGE_SESSION_KEY = getattr(translation, 'LANGUAGE_SESSION_KEY', 'django_language')
        request.session[LANGUAGE_SESSION_KEY] = user_language

    request.session.save()

    if request.session.test_cookie_worked():
        request.session.delete_test_cookie()


def redirect_logged_user(request, redirect_field_name):
    # Redirecionar usuário para a pagina desejada
    redirect_to = request.REQUEST.get(redirect_field_name, '')
    if not redirect_to or '//' in redirect_to or ' ' in redirect_to:
        redirect_to = settings.LOGIN_REDIRECT_URL

    return HttpResponseRedirect(redirect_to)


def handle_redirect_to(request, template_name, redirect_field_name, form, **kwargs):

    context = kwargs.get('extra_context', {})

    redirect_to = request.REQUEST.get(redirect_field_name, '')

    request.session.set_test_cookie()

    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = RequestSite(request)

    context.update({
        'form': form,
        redirect_field_name: redirect_to,
        'site': current_site,
        'site_name': current_site.name,
    })
    return render_to_response(
        template_name,
        context,
        context_instance=RequestContext(request)
    )



# -*- coding: utf-8 -*-
try:
    from django.conf.urls.defaults import *
except ImportError:
    from django.conf.urls import *

from django.contrib.auth import views as auth_views

from shortcuts import route
from views import login, show_login
from views import new_identity, register
from forms import IdentityAuthenticationForm, RegistrationForm

urlpatterns = patterns('identity_client.views',
    route(r'^registration/',
        GET=new_identity,
        POST=register,
        kwargs={'registration_form': RegistrationForm},
        name='registration_register'
    ),
    route(r'^login/$',
        GET=show_login,
        POST=login,
        kwargs={'authentication_form': IdentityAuthenticationForm},
        name='auth_login'
    ),
     url(r'^logout/$',
        auth_views.logout,
        {'template_name': 'logout.html'},
        name='auth_logout'
    ),
     url(r'^accounts/$',
        'list_accounts',
        name='list_accounts'
    ),
    (r'^sso/', include('identity_client.sso.urls', namespace='sso_consumer')),
)



# -*- coding: utf-8 -*-
"""
Source: https://github.com/brosner/django-timezones
Original file: https://github.com/brosner/django-timezones/blob/3ce983c23b50ee5277c6ba6c5b7af9952a281cd7/timezones/zones.py
"""
from datetime import datetime

import pytz

ALL_TIMEZONE_CHOICES = tuple(zip(pytz.all_timezones, pytz.all_timezones))
COMMON_TIMEZONE_CHOICES = tuple(zip(pytz.common_timezones, pytz.common_timezones))
PRETTY_TIMEZONE_CHOICES = []

for tz in pytz.common_timezones:
    now = datetime.now(pytz.timezone(tz))
    ofs = now.strftime("%z")
    PRETTY_TIMEZONE_CHOICES.append((int(ofs), tz, "(GMT%s) %s" % (ofs, tz)))
PRETTY_TIMEZONE_CHOICES.sort()
for i in xrange(len(PRETTY_TIMEZONE_CHOICES)):
    PRETTY_TIMEZONE_CHOICES[i] = PRETTY_TIMEZONE_CHOICES[i][1:]



#-*- coding: utf-8 -*-
import logging
from functools import wraps

import futures
import requests
from django import http
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import user_passes_test
from django.template import Context, loader

from identity_client.utils import get_account_module
from identity_client.utils import prepare_form_errors


__all__ = [
    'required_method', 'sso_login_required', 'requires_plan', 'with_403_page',
    'handle_api_exceptions', 'handle_api_exceptions_with_form',
]


class required_method(object):

    def __init__(self, *args, **kwargs):
        self.methods = args
        self.error = kwargs.get('error',
            # Default error:
            http.HttpResponse('Not Implemented',
                mimetype='text/plain', status=501)
        )

    def __call__(self, view):
        def f(request, *args, **kwargs):
            if request.method in self.methods:
                return view(request, *args, **kwargs)
            else:
                return self.error
        f.__name__ = view.__name__
        f.__doc__ = view.__doc__
        return f


def sso_login_required(view):

    def decorated(request, *args, **kwargs):
        url = reverse('sso_consumer:request_token')

        actual_decorator = user_passes_test(
            lambda user: user.is_authenticated(),
            login_url=url,
        )

        wrapped_view = actual_decorator(view)

        return wrapped_view(request, *args, **kwargs)

    return decorated


def requires_plan(plan_slug):

    def decorator(view):

        @sso_login_required
        def check_user_plan(request, *args, **kwargs):
            """
            Este decorator assume que a autenticação acabou de ser feita,
            portanto as contas do usuário estão sincronizadas com o Passaporte Web.
            """
            serviceAccount = get_account_module()
            user_accounts = serviceAccount.for_identity(request.user)
            required_plan_accounts = user_accounts.filter(plan_slug=plan_slug)

            if required_plan_accounts.count() > 0:
                return view(request, *args, **kwargs)
            else:
                logging.info(
                    'Request from user %s (uuid=%s) denied (%s). User has no accounts at plan "%s".',
                    request.user.email,
                    request.user.uuid,
                    request.path,
                    plan_slug,
                )
                return http.HttpResponseForbidden()

        return check_user_plan
        
    return decorator


def with_403_page(view):

    # You need to create a 403.html template.
    template_name = '403.html'

    def handle_403(request, *args, **kwargs):
        response = view(request, *args, **kwargs)

        if isinstance(response, http.HttpResponseForbidden):
            t = loader.get_template(template_name) 
            response = http.HttpResponseForbidden(
                t.render(
                    Context({'MEDIA_URL': settings.MEDIA_URL})
                )
            )

        return response

    return handle_403


def handle_api_exceptions(method):

    @wraps(method)
    def handler(cls, *args, **kwargs):
        status_code = 500
        content = error = None

        try:
            status_code, content = method(cls, *args, **kwargs)

        except requests.exceptions.HTTPError as e:
            status_code = e.response.status_code
            error = {
                'status': e.response.status_code,
                'message': e.response.text if e.response.text else e.message,
            }

        except requests.exceptions.ConnectionError as e:
            error = {
                'status': None,
                'message': 'Error connecting to PassaporteWeb',
            }

        except requests.exceptions.Timeout as e:
            error = {
                'status': None,
                'message': 'Timeout connecting to PassaporteWeb',
            }

        except (requests.exceptions.RequestException, Exception) as e:
            error = {
                'status': None,
                'message': u'Unexpected error: {0} <{1}>'.format(e, type(e)),
            }

        if error:
            logging.error(
                'fetch_identity_data: Error making request: %s - %s',
                error['status'], error['message']
            )

        return (status_code, content, error)

    return handler


def handle_api_exceptions_with_form(method):

    @wraps(method)
    def handler(cls, form, *args, **kwargs):
        status_code = 500
        content = error = error_dict = None

        try:
            status_code, content = method(cls, form, *args, **kwargs)

        except requests.exceptions.HTTPError as e:
            status_code = e.response.status_code

            if e.response.status_code in (400, 409):
                error_dict = e.response.json()
            else:
                error_dict = {
                    '__all__': [error_messages.get(e.response.status_code, error_messages['default']), ]
                }

            error = {
                'status': e.response.status_code,
                'message': e.response.text if e.response.text else e.message,
            }

        except requests.exceptions.ConnectionError as e:
            error_dict = {
                '__all__': [error_messages.get('ConnectionError', error_messages['default']), ]
            }
            error = {
                'status': None,
                'message': 'Error connecting to passaporteweb',
            }

        except requests.exceptions.Timeout as e:
            error_dict = {
                '__all__': [error_messages.get('Timeout', error_messages['default']), ]
            }
            error = {
                'status': None,
                'message': 'Timeout connecting to passaporteweb',
            }

        except (requests.exceptions.RequestException, Exception) as e:
            error_dict = {
                '__all__': [error_messages.get('default'), ]
            }
            error = {
                'status': None,
                'message': u'Unexpected error: {0} <{1}>'.format(e, type(e)),
            }

        if error_dict:
            form._errors = prepare_form_errors(error_dict)

        if error:
            logging.error(
                '%s: Error making request: %s - %s',
                method.__name__, error['status'], error['message']
            )

        return (status_code, content, form)

    return handler


def asyncmethod(method):

    def callback(future):
        try:
            result = future.result()
            return result
        except Exception as e:
            logging.error(
                'asyncmethod: Future "%s" raised an exception: "%s" (%s)',
                future, e, type(e)
            )

    @wraps(method)
    def handler(*args, **kwargs):
        logging.info(
            'asyncmethod: Method "%s" invoked with args "%s" and kwargs "%s"',
            method.__name__, args, kwargs
        )
        with futures.ThreadPoolExecutor(max_workers=1) as executor:
            future_result = executor.submit(method, *args, **kwargs)
            future_result.add_done_callback(callback)

        return future_result

    return handler


error_messages = {
    401: u"Esta aplicação não está autorizada a utilizar o PassaporteWeb. Entre em contato com o suporte.",
    400: u"Erro na transmissão dos dados. Tente novamente.",
    409: u"Email já cadastrado.",
    'ConnectionError': u"Ocorreu uma falha na comunicação com o Passaporte Web. Por favor tente novamente.",
    'Timeout': u"Ocorreu uma falha na comunicação com o Passaporte Web. Por favor tente novamente.",
    'default': u"Erro no servidor. Entre em contato com o suporte.",
}



# -*- coding: utf-8 -*-
import json

from mock import Mock, patch
import requests

from django.core.urlresolvers import reverse

import identity_client
from identity_client.client_api_methods import APIClient
from identity_client.tests.helpers import MyfcIDTestCase as TestCase


__all__ = ["IdentityRegistrationTest", "IdentityLoginTest"]


test_user_email = 'identity_client@disposableinbox.com'
test_user_password = '*SudN7%r$MiYRa!E'
test_user_uuid = 'c3769912-baa9-4a0c-9856-395a706c7d57'


class IdentityRegistrationTest(TestCase):

    def setUp(self):
        self.registration_data = {
            'first_name': 'Myfc ID',
            'last_name': 'Clients',
            'email': test_user_email,
            'password': test_user_password,
            'password2': test_user_password,
            'tos': True,
        }

        self.url = reverse('registration_register')


    def test_successful_api_registration(self):
        with identity_client.tests.use_cassette('invoke_registration_api/registration_success'):
            response = self.client.post(self.url, self.registration_data)

        self.assertEquals(302, response.status_code)
        self.assertEquals(self.client.session['user_data'], {
            u'first_name': u'Myfc ID',
            u'last_name': u'Clients',
            u'send_partner_news': False,
            u'uuid': u'c3769912-baa9-4a0c-9856-395a706c7d57',
            u'is_active': False,
            u'cpf': None,
            u'update_info_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/',
            u'notifications': {u'count': 0, u'list': u'/notifications/api/'},
            u'accounts': [],
            u'send_myfreecomm_news': False,
            u'services': {u'identity_client': u'/accounts/api/service-info/c3769912-baa9-4a0c-9856-395a706c7d57/identity_client/'},
            u'email': u'identity_client@disposableinbox.com',
            u'profile_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/profile/'
        })


    def test_conflict_error_on_api_registration(self):
        with identity_client.tests.use_cassette('invoke_registration_api/email_already_registered'):
            response = self.client.post(self.url, self.registration_data)

        form = response.context['form']
        self.assertEquals(form.errors, {
            u'field_errors': [u'email'],
            u'email': [u'Este email já está cadastrado. Por favor insira outro email']
        })


    @patch.object(APIClient, 'invoke_registration_api', Mock())
    def test_form_renderization_because_of_empty_fields(self):
        empty_post_data = {'password':'', 'password2':'', 'email':''}
        response = self.client.post(self.url, empty_post_data)
        self.assertFalse(APIClient.invoke_registration_api.called)


class IdentityLoginTest(TestCase):

    def test_successful_login(self):
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            response = self.client.post(
                reverse('auth_login'), {
                    'email': test_user_email,
                    'password': test_user_password,
                }
            )
        self.assertTrue('_auth_user_id' in self.client.session)


    def test_add_userdata_to_session_after_login(self):
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            response = self.client.post(
                reverse('auth_login'), {
                    'email': test_user_email,
                    'password': test_user_password,
                }
            )
        self.assertEquals(self.client.session['user_data'], {
            u'authentication_key': u'$2a$12$nA3ad2y5aSBlg80K9ekbNuvnRO1OI1WUKZyoJqWEhk.PQpD8.6jkS',
            u'email': u'identity_client@disposableinbox.com',
            u'first_name': u'Identity',
            u'id_token': u'729dd3a15cf03a80024d0986deee9ae91fdd5d834fabf6f9',
            u'is_active': True,
            u'last_name': u'Client',
            u'notifications': {u'count': 0, u'list': u'/notifications/api/'},
            u'profile_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/profile/',
            u'send_myfreecomm_news': True,
            u'send_partner_news': True,
            u'update_info_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/',
            u'uuid': u'c3769912-baa9-4a0c-9856-395a706c7d57'
        })



# -*- coding: utf-8 -*-
from helpers import use_cassette

from test_backend import *
from test_forms import *
from test_models import *
from test_client_views import *
from test_sso_client import *
from test_sso_views import *
from test_p3p_middleware import *
from test_client_api_methods import *
from test_decorators import *



# -*- coding: utf-8 -*-
import logging
import json
from httplib2 import HttpLib2Error

from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponse
from django.http import HttpResponseServerError
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.auth import REDIRECT_FIELD_NAME

from identity_client.sso.client import SSOClient
from identity_client.sso.decorators import oauth_callback
from identity_client.backend import MyfcidAPIBackend
from identity_client.views import login_user


__all__ = ['initiate', 'fetch_user_data']


def handle_api_exception(view):
    def func(*args, **kwargs):
        try:
            return view(*args, **kwargs)
        except HttpLib2Error, e:
            logging.error(
                '%s: Error during http request: %s<%s>',
                view.__name__,  e, type(e)
            )
            return HttpResponseServerError(status=502)
        except (AssertionError, ), e:
            logging.error(
                '%s: Unexpected status code %s<%s>',
                view.__name__, e, type(e)
            )
            return HttpResponseServerError()
    func.__name__ = view.__name__
    func.__doc__  = view.__doc__
    return func


def render_sso_iframe(request):
    context = {
        'myfcid_host': settings.PASSAPORTE_WEB['HOST'],
        'application_host': settings.APPLICATION_HOST,
        'application_slug': settings.PASSAPORTE_WEB['SLUG'],
    }

    return render_to_response(
        'sso_iframe.html',
        context,
        context_instance=RequestContext(request)
    )


@handle_api_exception
def initiate(request):

    try:
        authorization_url = SSOClient().authorize(request)

        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)
        logging.debug("Session %s data: %s", session_key, request.session.items())

    except AssertionError, e:
        resp, content = e.args

        message = "Could not fetch request token. Response was {0} - {1}".format(
            resp.get('status'), content
        )
        logging.error(message)
        return HttpResponseServerError(content=message)

    except ValueError, e:
        message = "Invalid request token"
        logging.error(message)
        return HttpResponseServerError(content=message)

    return HttpResponseRedirect(authorization_url)


@handle_api_exception
@oauth_callback
def fetch_user_data(request):

    try:
        resp, raw_user_data = SSOClient(request.access_token).post(SSOClient.user_data_url)

        if not str(resp.get('status')) == '200':
            raise AssertionError(resp, raw_user_data)

        identity = json.loads(
            raw_user_data, object_hook=as_local_identity
        )
        login_user(request, identity)

    except AssertionError, e:
        resp, content = e.args

        message = "Could not fetch user data. Response was {0} - {1}".format(
            resp.get('status'), content
        )
        logging.error(message)
        return HttpResponseServerError(content=message)

    except ValueError, e:
        message = "Invalid user data: {0}".format(raw_user_data)
        logging.error(message)
        return HttpResponseServerError(content=message)

    next_url = request.session.get(REDIRECT_FIELD_NAME, settings.LOGIN_REDIRECT_URL)

    return HttpResponseRedirect(next_url)


def as_local_identity(data):

    if ('uuid' in data) and ('email' in data):
        return MyfcidAPIBackend().create_local_identity(data)

    return data
